@@ -1,7 +1,7 @@
 package Config::IOD::Document;
 
-our $DATE = '2015-03-12'; # DATE
-our $VERSION = '0.10'; # VERSION
+our $DATE = '2015-03-18'; # DATE
+our $VERSION = '0.11'; # VERSION
 
 use 5.010;
 use strict;
@@ -57,6 +57,12 @@ sub new {
     bless \%attrs, $class;
 }
 
+sub empty {
+    my $self = shift;
+    $self->_discard_cache;
+    $self->{_parsed} = [];
+}
+
 # all _validate_*() methods return ($err_msg, $validated_val)
 
 sub _validate_section {
@@ -102,6 +108,96 @@ sub _blank_line {
     ["B", "\n"];
 }
 
+# cache is used for get_value() and get_raw_value() to avoid re-scanning the
+# files on every invocation. but whenever one of document-modifying methods is
+# called, we discard the cache
+sub _discard_cache {
+    my $self = shift;
+    delete $self->{_dump_cache};
+}
+
+sub dump {
+    my $self = shift;
+
+    my $linum = 0;
+    my $merge;
+    my $cur_section = $self->{_parser}{default_section};
+    my $res = {};
+    my $arrayified = {};
+    my $num_seen_section_lines = 0;
+
+    my $_merge = sub {
+        return if $cur_section eq $merge;
+        croak "IOD document:$linum: Can't merge section '$merge' to ".
+            "'$cur_section': Section '$merge' not seen yet"
+                unless exists $res->{$merge};
+        for my $k (keys %{ $res->{$merge} }) {
+            $res->{$cur_section}{$k} //= $res->{$merge}{$k};
+        }
+    };
+
+    for my $line (@{ $self->{_parsed} }) {
+        $linum++;
+        my $type = $line->[COL_TYPE];
+        if ($type eq 'D') {
+            my $directive = $line->[COL_D_DIRECTIVE];
+            if ($directive eq 'merge') {
+                my $args = $self->{_parser}->_parse_command_line(
+                    $line->[COL_D_ARGS_RAW]);
+                if (!defined($args)) {
+                    croak "IOD document:$linum: Invalid arguments syntax '".
+                        $line->[COL_D_ARGS_RAW]."'";
+                }
+                $merge = @$args ? $args->[0] : undef;
+            } # ignore the other directives
+        } elsif ($type eq 'S') {
+            $num_seen_section_lines++;
+            # merge previous section
+            $_merge->() if defined($merge) && $num_seen_section_lines > 1;
+            $cur_section = $line->[COL_S_SECTION];
+            $res->{$cur_section} //= {};
+        } elsif ($type eq 'K') {
+            # the common case is that value are not decoded or
+            # quoted/bracketed/braced, so we avoid calling _parse_raw_value here
+            # to avoid overhead
+            my $key = $line->[COL_K_KEY];
+            my $val = $line->[COL_K_VALUE_RAW];
+            if ($val =~ /\A["!\\[\{]/) {
+                my ($err, $parse_res, $decoded_val) =
+                    $self->{_parser}->_parse_raw_value($val);
+                croak "IOD document:$linum: Invalid value: $err" if $err;
+                $val = $decoded_val;
+            } else {
+                $val =~ s/\s*[#;].*//; # strip comment
+            }
+
+            if (exists $res->{$cur_section}{$key}) {
+                if (!$self->{_parser}{allow_duplicate_key}) {
+                    croak "IOD document:$linum: Duplicate key: $key ".
+                        "(section $cur_section)";
+                } elsif ($arrayified->{$cur_section}{$key}++) {
+                    push @{ $res->{$cur_section}{$key} }, $val;
+                } else {
+                    $res->{$cur_section}{$key} = [
+                        $res->{$cur_section}{$key}, $val];
+                }
+            } else {
+                $res->{$cur_section}{$key} = $val;
+            }
+        } # ignore the other line types
+    }
+
+    $_merge->() if defined($merge) && $num_seen_section_lines > 1;;
+
+    $res;
+}
+
+sub get_value {
+    my ($self, $section, $key) = @_;
+    $self->{_dump_cache} = $self->dump unless $self->{_dump_cache};
+    $self->{_dump_cache}{$section}{$key};
+}
+
 sub _find_section {
     my $self = shift;
     my $opts;
@@ -267,6 +363,8 @@ sub insert_section {
         $opts->{comment}, # COL_S_COMMENT
         "\n", # COL_S_NL
     ];
+
+    $self->_discard_cache;
     $linum;
 }
 
@@ -349,6 +447,7 @@ sub insert_key {
         $value, # COL_K_VALUE_RAW
         "\n", # COL_K_NL
     ];
+    $self->_discard_cache;
     $linum;
 }
 
@@ -382,6 +481,7 @@ sub delete_section {
         splice @$p, $line1-1, ($line2-$line1+1);
         $num_deleted++;
     }
+    $self->_discard_cache if $num_deleted;
     $num_deleted;
 }
 
@@ -414,6 +514,8 @@ sub delete_key {
         splice @$p, $linum-1, 1;
         $num_deleted++;
     }
+
+    $self->_discard_cache if $num_deleted;
     $num_deleted;
 }
 
@@ -495,7 +597,7 @@ Config::IOD::Document - Represent IOD document
 
 =head1 VERSION
 
-This document describes version 0.10 of Config::IOD::Document (from Perl distribution Config-IOD), released on 2015-03-12.
+This document describes version 0.11 of Config::IOD::Document (from Perl distribution Config-IOD), released on 2015-03-18.
 
 =head1 SYNOPSIS
 
@@ -667,6 +769,26 @@ occurrence.
 
 =back
 
+=head2 $doc->dump() => hoh
+
+Return a hoh (hash of section names and hashes, where each of the second-level
+hash is of keys and values), Values will be decoded and merging will be done,
+but includes are not processed (even though C<include> directive is active).
+
+=head2 $doc->empty()
+
+Empty document.
+
+=head2 $doc->get_value($section, $key) => $value
+
+Get value. Values are decoded and section merging is respected, but includes are
+not processed.
+
+Internally, will do a C<dump()> and cache the result so subsequent
+C<get_value()> will avoid re-parsing the whole document. (The cache will
+automatically be discarded is one of document-modifying methods like
+C<delete_section()> is called.)
+
 =head1 SEE ALSO
 
 L<Config::IOD>
